/* Lesson 11.5 Demo 2 - Use the MERGE Statement to Merge Source Data to a Target */
/*
Create two temporary tables:
	 Original_Cast (source)
     Updated_Cast (target)
Update data in the Original_Cast (source) table
Use the MERGE statement to update the Updated_Cast (target) table
*/

USE TempDB;
GO
 
IF OBJECT_ID ('tempdb..#Original_Cast') IS NOT NULL DROP TABLE #Original_Cast;
IF OBJECT_ID ('tempdb..#Updated_Cast')  IS NOT NULL DROP TABLE #Updated_Cast;
 
CREATE TABLE #Original_Cast
(  CastID    TINYINT NOT NULL
  ,Name   VARCHAR (25) NULL
  ,Location         VARCHAR (25) NULL);
GO
 
CREATE TABLE #Updated_Cast
(  CastID    TINYINT NOT NULL
  ,Name   VARCHAR (25) NULL
  ,Location         VARCHAR (25) NULL);
GO
 
INSERT INTO #Original_Cast (CastID, Name, Location)
   VALUES (1, 'Cinderella', 'Fantasyland')
         ,(2, 'Aladdin', 'Adventureland')
         ,(3, 'Brer Rabbit', 'Frontierland')
		 ,(4, 'Charming', 'Fantasyland');

INSERT INTO #Updated_Cast (CastID, Name, Location)
   VALUES (1, 'Cinderella', 'Fantasyland')
         ,(2, 'Aladdin', 'Adventureland')
         ,(3, 'Brer Rabbit', 'Frontierland')
		 ,(4, 'Charming', 'Fantasyland');

SELECT *
FROM #Original_Cast
GO

SELECT *
FROM #Updated_Cast
GO

--UPDATE, INSERT, and DELETE entries in the Original_Cast table --
-- Update Mister Charming's Title --
UPDATE #Original_Cast
   SET Name = 'Prince Charming'
 WHERE CastID = 4
-- Delete Brer Rabbit --
DELETE #Original_Cast
 WHERE CastID = 3
--Add Buzz Lightyear and his location --
INSERT INTO #Original_Cast (CastID, Name, Location)
VALUES (5, 'Buzz Lightyear', 'Tomorrowland')
GO

SELECT *
FROM #Original_Cast
GO

SELECT *
FROM #Updated_Cast
GO

-- Using the Original_Cast table as the source, MERGE changes into the Updated_Cast table --
MERGE  #Updated_Cast AS Target
 USING #Original_Cast AS Source
    ON Target.CastID = Source.CastID
WHEN MATCHED
                AND (Target.Name <> Source.Name
                     OR Target.Location <> Source.Location)
THEN
   UPDATE SET    -- this UPDATES Prince Charming's name --
      Target.Name = Source.Name
     ,Target.Location = Source.Location
WHEN NOT MATCHED BY TARGET  
THEN
   INSERT (CastID, Name, Location)  -- this INSERTS Buzz into the Updated_Cast table --
   VALUES (Source.CastID, Source.Name, Source.Location)
WHEN NOT MATCHED BY SOURCE THEN DELETE;  -- for this sample, just delete any unmatched data --
GO

SELECT *
FROM #Original_Cast
GO

SELECT *
FROM #Updated_Cast
GO

--Cleanup Tables --
DROP TABLE #Original_Cast;
DROP TABLE #Updated_Cast;