/* Lesson 14.7 Demo 2 - Use Dynamic Management Objects */


-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
------Following script might be helpful in Dev environment----------------------
-->>>>>>if you need to eliminate older cached plans<<<<<<------------------------
-----However, not always beneficial on production server------------------------
--clearing the cache has an associated cost since new plans need to be created-- 

-->>>>>>Clearing the cache for the AdventureWorks2012 database<<<<<<--
--Determine the id of the AdventureWorks2012 database
DECLARE @intDBID INTEGER
SET @intDBID = ( SELECT dbid
FROM master.dbo.sysdatabases
WHERE name = 'AdventureWorks2012'
)
--Flush the procedure cache of plans belonging to the AdventureWorks2012 database
DBCC FLUSHPROCINDB (@intDBID)

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


-->>>>>>Using DMO to retrieve plans for compiled objects <<<<<<---------
--Large amount of plan reuse generally exhibits a healthy system--------
--If there are many plans whose usecount = 1, means a lot of ad hoc queries --
USE AdventureWorks2012;
GO

SELECT refcounts AS 'Number of cache objects referencing'
 ,usecounts AS 'Number of times cache object has been looked up'
 ,size_in_bytes
 ,cacheobjtype AS 'Type of cache object'
 ,objtype
FROM sys.dm_exec_cached_plans
WHERE objtype IN ( 'proc', 'prepared' ) ;

-->>>>>>Using DMO to retrieve total number of plans in cache<<<<<<-------
USE AdventureWorks2012;
GO

SELECT COUNT(*) AS 'Total Number of Cached Plans'
FROM sys.dm_exec_cached_plans ;

-->>>>>>Even better idea of reuse is to get a distribution<<<<<<--------
USE AdventureWorks2012;
GO

SELECT MAX(CASE WHEN usecounts BETWEEN 10 AND 100 THEN '10-100'
WHEN usecounts BETWEEN 101 AND 1000 THEN '101-1000'
WHEN usecounts BETWEEN 1001 AND 5000 THEN '1001-5000'
WHEN usecounts BETWEEN 5001 AND 10000 THEN '5001-10000'
ELSE CAST(usecounts AS VARCHAR(100))
END) AS usecounts ,
COUNT(*) AS countInstance
FROM sys.dm_exec_cached_plans
GROUP BY CASE WHEN usecounts BETWEEN 10 AND 100 THEN 50
WHEN usecounts BETWEEN 101 AND 1000 THEN 500
WHEN usecounts BETWEEN 1001 AND 5000 THEN 2500
WHEN usecounts BETWEEN 5001 AND 10000 THEN 7500
ELSE usecounts
END
ORDER BY CASE WHEN usecounts BETWEEN 10 AND 100 THEN 50
WHEN usecounts BETWEEN 101 AND 1000 THEN 500
WHEN usecounts BETWEEN 1001 AND 5000 THEN 2500
WHEN usecounts BETWEEN 5001 AND 10000 THEN 7500
ELSE usecounts
END DESC ;

--The following returns the SQL text of all cached entries that have been executed more than once
USE AdventureWorks2012;
GO

SELECT usecounts, cacheobjtype, objtype, text 
FROM sys.dm_exec_cached_plans 
CROSS APPLY sys.dm_exec_sql_text(plan_handle) 
WHERE usecounts >1  
ORDER BY usecounts DESC;
GO

/*
Returning the SET options with which the plan was compiled
The following returns the SET options with which the plan was compiled 
The sql_ handle for the plan is also returned. 
The PIVOT operator is used to output the set_ options and sql_ handle attributes as columns 
rather than as rows. 
*/
USE AdventureWorks2012;
GO

SELECT plan_handle, pvt.set_options, pvt.sql_handle
FROM (
      SELECT plan_handle, epa.attribute, epa.value 
      FROM sys.dm_exec_cached_plans 
      OUTER APPLY sys.dm_exec_plan_attributes(plan_handle) AS epa
      WHERE cacheobjtype = 'Compiled Plan'
      ) AS ecpa 
PIVOT (MAX(ecpa.value) FOR ecpa.attribute IN ("set_options", "sql_handle")) AS pvt;
GO

-->>>>>>Returning and examining the top 10 most reused plans<<<<<<----------
SELECT TOP 10 WITH TIES
decp.usecounts ,
decp.cacheobjtype ,
decp.objtype ,
deqp.query_plan ,
dest.text
FROM sys.dm_exec_cached_plans decp
CROSS APPLY sys.dm_exec_query_plan(decp.plan_handle) AS deqp
CROSS APPLY sys.dm_exec_sql_text(decp.plan_handle) AS dest
ORDER BY usecounts DESC ;