/* Lesson 17.7 Demo - Using XACT_ABORT */

--------------------------------------------------------------
-->>>>>>First Query processes with XACT_ABORT OFF<<<<<<--
USE AdventureWorks2012;
GO

SET XACT_ABORT OFF;
PRINT 'Starting Query with XACT_ABORT OFF';
SET IDENTITY_INSERT HumanResources.Department ON;
INSERT INTO HumanResources.Department
           (DepartmentID
		   ,Name
           ,GroupName
           ,ModifiedDate)
     VALUES
           (1
           ,N'Duplicate DepartmentID'
		   ,N'Research'
           ,2014-06-01);
SET IDENTITY_INSERT HumanResources.Department OFF;
PRINT 'Error Occurred';
GO
PRINT 'Starting New Batch';
SET XACT_ABORT OFF;
GO

------------------------------------------------------------
-->>>>>>Second Query processes with XACT_ABORT ON<<<<<<--
USE AdventureWorks2012;
GO

SET XACT_ABORT ON;
PRINT 'Starting Query with XACT_ABORT ON';
SET IDENTITY_INSERT HumanResources.Department ON;
INSERT INTO HumanResources.Department
           (DepartmentID
		   ,Name
           ,GroupName
           ,ModifiedDate)
     VALUES
           (1
           ,N'Duplicate DepartmentID'
		   ,N'Research'
           ,2014-06-01);
SET IDENTITY_INSERT HumanResources.Department OFF;
PRINT 'Error Occurred';
GO
PRINT 'Starting New Batch';
SET XACT_ABORT OFF;
GO

-----------------------------------------------------------------------
-->>>>>>Third query uses THROW with XACT_ABORT ON <<<<<<--
-->>>>>>Batch is aborted even though THROW is included in code <<<<<<--
USE AdventureWorks2012;
GO

SET XACT_ABORT ON;
PRINT 'Starting Query with XACT_ABORT ON';

SET IDENTITY_INSERT HumanResources.Department ON;
INSERT INTO HumanResources.Department
           (DepartmentID
		   ,Name
           ,GroupName
           ,ModifiedDate)
     VALUES
           (1
           ,N'Duplicate DepartmentID'
		   ,N'Research'
           ,2014-06-01);
SET IDENTITY_INSERT HumanResources.Department OFF;

THROW 50000, 'An Error Occurred', 0;
PRINT 'After Error Occurred';
GO
PRINT 'Starting New Batch';
SET XACT_ABORT OFF;
GO

---------------------------------------------------------------------
-->>>>>>Fourth query uses TRY/CATCH <<<<<<--
-->>>>>>Using XACT_ABORT in TRY/CATCH<<<<<<--
USE AdventureWorks2012;
GO

BEGIN TRY
    BEGIN TRANSACTION;
		-->> SET XACT_ABORT ON renders transaction uncommittable
		-->> when constraint violation occurs
		SET XACT_ABORT ON;
        -->> FOREIGN KEY constraint exists on this table
        -->> Satement will generate constraint violation error
        DELETE FROM Production.Product
            WHERE ProductID = 328;
		-->> If delete operation succeeds, transaction committed
		-->> and CATCH block will not execute
    COMMIT TRANSACTION;
END TRY
BEGIN CATCH
        -->> Test if transaction uncommittable
    IF (XACT_STATE()) = -1
    BEGIN
        PRINT 'Transaction cannot be committed' +
              ' Rolling back transaction'
        ROLLBACK TRANSACTION;
    END;

    -->> Test if transaction is active
    IF (XACT_STATE()) = 1
    BEGIN
        PRINT 'Transaction is committable' + 
              ' Committing transaction'
        COMMIT TRANSACTION;   
    END;
END CATCH;
GO
SET XACT_ABORT OFF;
GO

----------------------------------------------------------
-->>>>>> Test state of XACT_ABORT <<<<<<--
USE AdventureWorks2012;
GO

DECLARE @XACT_ABORT VARCHAR(3) = 'OFF';
IF ( (16384 & @@OPTIONS) = 16384 ) SET @XACT_ABORT = 'ON';
SELECT @XACT_ABORT AS XACT_ABORT;
