/* Lesson 4 INLINE Code */

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 4.2 Create and Use AFTER Triggers----------------------

USE AdventureWorks2012
GO

IF OBJECT_ID (N'Sales.tr_CustomerReminder', N'TR') IS NOT NULL
   DROP TRIGGER Sales.tr_CustomerReminder;
GO
CREATE TRIGGER Sales.tr_CustomerReminder
ON Sales.Customer
AFTER INSERT, UPDATE 
AS RAISERROR ('Notify Customer Relations of Updated Information', 16, 10);
GO


IF OBJECT_ID (N'Sales.tr_CustomerReminderEmail',N'TR') IS NOT NULL
    DROP TRIGGER Sales.tr_CustomerReminderEmail;
GO
CREATE TRIGGER Sales.tr_CustomerReminderEmail
ON Sales.Customer
AFTER INSERT, UPDATE, DELETE 
AS
   EXEC msdb.dbo.sp_send_dbmail
        @profile_name = 'AdventureWorks2012 Administrator',
        @recipients = 'marilyn@Adventure-Works.com',
        @body = 'Don''t forget to print a report for the sales force.',
        @subject = 'Reminder';
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 4.4 Define Nested AFTER Triggers---------------------------
IF ( (SELECT TRIGGER_NESTLEVEL( OBJECT_ID('Production.tr_ProductionCategories_CategoryName') , 'AFTER' , 'DML' ) ) > 5 )
   RAISERROR('Trigger Production.tr_ProductionCategories_CategoryName nested more than 5 levels.',16,-1)

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 4.5 INSTEAD OF Trigger-------------------------------------
USE CertData
GO
/* Delete Trigger if already created in table */
IF OBJECT_ID (N'Production.tr_ProductionCategories_CategoryNameCheck', N'TR') IS NOT NULL
   DROP TRIGGER Production.tr_ProductionCategories_CategoryNameCheck;
GO

/* CREATE INSTEAD OF Trigger */
CREATE TRIGGER Production.tr_ProductionCategories_CategoryNameCheck
ON Production.Categories
INSTEAD OF INSERT
AS
BEGIN
  SET NOCOUNT ON;            --count of number of rows affected not returned --
  IF EXISTS (SELECT COUNT(*)
        FROM Inserted AS I
        JOIN Production.Categories AS C
          ON I.CategoryName = C.CategoryName
		GROUP BY I.CategoryName
		HAVING COUNT(*) > 0 )  --check if Inserted CategoryName is already in Production.Categories --
		THROW 50000, 'Duplicate category names are not allowed', 0; --display error message --
	ELSE
	BEGIN
		INSERT Production.Categories (CategoryName, Description) -- if not duplicate INSERT the category name --
		SELECT CategoryName, Description FROM Inserted;
	END;
END;
GO



/* Test the INSTEAD OF Trigger  - INSERT a category name That already exists in the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('MCSA SQL Server 2012', 'New Category Description');
GO

/* Test the INSTEAD OF Trigger - INSERT a new category name the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('New Category Name2', 'New Category Description');
GO

/* Cleanup by deleting Trigger */
IF OBJECT_ID (N'Production.tr_ProductionCategories_CategoryNameCheck', N'TR') IS NOT NULL
   DROP TRIGGER Production.tr_ProductionCategories_CategoryNameCheck;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-- Lesson 4.6 Use Trigger Update Functions---------------------------------
USE CertData
GO

-- Example of DML Trigger Function referenced by an INSERT or UPDATE statement --

IF UPDATE(Price)
BEGIN    
	PRINT 'Column Price is being updated'    
-- do the required processing here 
END;

--The UPDATE statement below would make the UPDATE(Price) Function True --
UPDATE Production.Products
	SET Price = 200
	WHERE ProductID = 461;

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Sample Trigger UPDATE function from MSDN Using Adventureworks2012
USE AdventureWorks2012;
GO
IF EXISTS (SELECT name FROM sys.objects
      WHERE name = 'reminder' AND type = 'TR')
   DROP TRIGGER Person.reminder;
GO
CREATE TRIGGER reminder
ON Person.Address
AFTER UPDATE 
AS 
IF ( UPDATE (StateProvinceID) OR UPDATE (PostalCode) )
BEGIN
RAISERROR (50009, 16, 10)
END;
GO
-- Test the trigger.
UPDATE Person.Address
SET PostalCode = 99999
WHERE PostalCode = '12345';
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 4.7 Handle Multiple Rows in a Session
-- Trigger is valid for single-row inserts.
USE AdventureWorks2012;
GO
CREATE TRIGGER NewPODetail
ON Purchasing.PurchaseOrderDetail
AFTER INSERT AS
   UPDATE PurchaseOrderHeader
   SET SubTotal = SubTotal + LineTotal
   FROM inserted
   WHERE PurchaseOrderHeader.PurchaseOrderID = inserted.PurchaseOrderID ;

----------------------------------------------------------------------------------
-- Trigger is valid for multirow and single-row inserts---------------------------
USE AdventureWorks2012;
GO
CREATE TRIGGER NewPODetail2
ON Purchasing.PurchaseOrderDetail
AFTER INSERT AS
   UPDATE Purchasing.PurchaseOrderHeader
   SET SubTotal = SubTotal + 
      (SELECT SUM(LineTotal)
      FROM inserted
      WHERE PurchaseOrderHeader.PurchaseOrderID
       = inserted.PurchaseOrderID)
   WHERE PurchaseOrderHeader.PurchaseOrderID IN
      (SELECT PurchaseOrderID FROM inserted);



