/* Lesson 4.3.Demo 1 Use AFTER Trigger */

/* Using the CertData database, 
   create an AFTER trigger that enforces uniqueness
   on the CategoryName column of the Categories table
*/

USE CertData
GO

/* Delete Trigger if already created in table */
IF OBJECT_ID (N'Production.tr_ProductionCategories_CategoryName', N'TR') IS NOT NULL
   DROP TRIGGER Production.tr_ProductionCategories_CategoryName;
GO

/* CREATE the AFTER Trigger */
CREATE TRIGGER Production.tr_ProductionCategories_CategoryName
ON Production.Categories
AFTER INSERT, UPDATE
AS
BEGIN
  IF @@ROWCOUNT = 0 RETURN;  --checks if no rows affected (used to improve performance) --
  SET NOCOUNT ON;            --count of number of rows affected not returned --
  IF EXISTS (SELECT COUNT(*)
        FROM Inserted AS I
        JOIN Production.Categories AS C
          ON I.CategoryName = C.CategoryName
		GROUP BY I.CategoryName
		HAVING COUNT(*) > 1 )  --check if Inserted CategoryName is already in Production.Categories --
    BEGIN
      THROW 50000, 'Duplicate category names are not allowed', 0; --display error message --
    END;
END;
GO

/* INSERT a new category name */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('NewCategory', 'New Category Description');
GO

SELECT * 
FROM Production.Categories;
GO

/* UPDATE the existing MCSE Data Platform CategoryName with NewCategory */
UPDATE Production.Categories
  SET CategoryName = 'MCSE Data Platform' WHERE CategoryName = 'NewCategory';
GO

SELECT * 
FROM Production.Categories;
GO

/* Clean up the Production.Categories Table */
DELETE FROM Production.Categories 
WHERE categoryname = 'NewCategory';
GO

SELECT * 
FROM Production.Categories;
GO

/* If desired, delete trigger */
IF OBJECT_ID (N'Production.tr_ProductionCategories_CategoryName', N'TR') IS NOT NULL
   DROP TRIGGER Production.tr_ProductionCategories_CategoryName;
GO