/* Lesson 5 INLINE Code */

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 5.7 Create Deterministic Query Results------

USE AdventureWorks2012
GO

--> Deterministic query results <--

SELECT AVG(ListPrice) AS 'Average List Price'
FROM Production.Product;
GO


--> Non-deterministic query results <--

SELECT GETDATE() AS 'Today''s Date'
FROM Production.Product;
GO

SELECT AVG(ListPrice) AS 'Average List Price', GETDATE() AS 'Today''s Date'
FROM Production.Product;
GO
 -->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 ------Lesson 5.11 Write Complex Queries - Joins versus Derived Tables---

USE AdventureWorks2012
GO
/* This example uses a derived table, 
a SELECT statement after the FROM clause, 
to return the last names of all employees and the cities in which they live
*/
SELECT p.LastName AS Name, d.City
FROM Person.Person AS p
INNER JOIN HumanResources.Employee e ON p.BusinessEntityID = e.BusinessEntityID 
INNER JOIN
   (SELECT ba.BusinessEntityID, a.City 
    FROM Person.Address AS a
    INNER JOIN Person.BusinessEntityAddress AS ba
    ON a.AddressID = ba.AddressID) AS d
ON p.BusinessEntityID = d.BusinessEntityID
ORDER BY p.LastName;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--Lesson 5.13 Access Data Using CASE, COALESCE, and ISNULL-----------------

----------------------CASE expression-------------------
USE AdventureWorks2012;
GO
-- Simple form of CASE expression --
-- Use SalariedFlag to designate employment status --
SELECT BusinessEntityID,LoginID,SalariedFlag,
CASE SalariedFlag
	WHEN 0 THEN 'Contractor'
	WHEN 1 THEN 'Employee'
	ELSE 'Trainee'
END AS 'Employment Status'
FROM HumanResources.Employee;
GO

USE AdventureWorks2012;
GO
-- Searched form of CASE expression --
--Defining the Shipmethod fro the difference of the due date and ship date
SELECT SalesOrderID, OrderDate, DueDate, ShipDate,
  CASE
   WHEN DATEDIFF(day, ShipDate, DueDate) < 3 then 'Expedite'
   WHEN DATEDIFF(day, ShipDate,DueDate) < 7 then 'Express'
   ELSE 'Standard'
  END AS ShipMethod
FROM Sales.SalesOrderHeader;
GO

------------------COALESCE----------------------------

USE AdventureWorks2012;
GO
--Original Query --
SELECT Name, Class, Color, ProductNumber
FROM Production.Product;
GO

USE AdventureWorks2012
GO
--USE COALESCE function --
--Returns first non-NULL value in set --
-- Data type is the argument with the highest data type precedence --
SELECT Name, Class, Color, ProductNumber,
COALESCE(Class, Color, ProductNumber) AS FirstNotNull
FROM Production.Product;
GO

----------------------------ISNULL--------------------------------------------------

USE AdventureWorks2012;
GO
-- USE ISNULL - Replace the NULL in MaxQty column query results with 0.00 --
--Data type is the argument of the first input --
SELECT Description, MinQty AS 'Min Quantity', ISNULL(MaxQty, 0.00) AS 'Max Quantity'
FROM Sales.SpecialOffer;
GO

USE AdventureWorks2012;
GO
--USE ISNULL - Replace the NULL in Size column in quer results with N/A --
--Data type is the argument of the first input --
SELECT ProductID, Name, ProductNumber, ISNULL(Size, N'N/A') AS 'Product Size'
FROM Production.Product;
GO
