/* Lesson 8.4 Demo 1 Group Data in Queries */

USE AdventureWorks2012
GO

/* Lesson 8.1: Use a Single Grouping Set - INLINE */
SELECT COUNT(*) AS 'Number of Products'
	FROM Production.ProductSubcategory;
GO

SELECT ProductCategoryID, COUNT(*) AS 'Number of Products per Category'
	FROM Production.ProductSubcategory
	GROUP BY ProductCategoryID;
GO
----------------------------------------------------------------------------------------------
--GROUPING BY VendorID - using COUNT(*), SUM, MAX --------------------------------------------
----------------------------------------------------------------------------------------------
SELECT VendorID, COUNT(*) AS 'Number of Orders', SUM(TotalDue) AS 'Total Cost of Purchases', MAX(OrderDate) AS 'Date of Last Order'
	FROM Purchasing.PurchaseOrderHeader
	GROUP BY VendorID 
	ORDER BY VendorID
GO

----------------------------------------------------------------------------------------------
--Same as previous query - ORDER BY the alias 'Total Cost of Purchases' from highest to lowest
----------------------------------------------------------------------------------------------
SELECT VendorID, COUNT(*) AS 'Number of Orders', SUM(TotalDue) AS 'Total Cost of Purchases', MAX(OrderDate) AS 'Date of Last Order'
	FROM Purchasing.PurchaseOrderHeader
	GROUP BY VendorID 
	ORDER BY 'Total Cost of Purchases' DESC;
GO

------------------------------------------------------------------------------------------------
--Using Multiple Elements in a Grouping Set ----------------------------------------------------
------------------------------------------------------------------------------------------------
SELECT VendorID, YEAR(OrderDate) AS 'Order Year', COUNT(*) AS 'Number of Orders', SUM(TotalDue) AS 'Total Cost of Purchases'
	FROM Purchasing.PurchaseOrderHeader
	GROUP BY VendorID, YEAR(OrderDate)  
	ORDER BY VendorID, 'Order Year';
GO

------------------------------------------------------------------------------------------------
--Use Grouping Functions to Aggregate Price ----------------------------------------------------
------------------------------------------------------------------------------------------------
SELECT TerritoryID, AVG(Freight) AS 'Average Shipping Charge'
	FROM Sales.SalesOrderHeader
	GROUP BY TerritoryID
	ORDER BY TerritoryID;
GO

------------------------------------------------------------------------------------------------
--Use Grouping Functions to Aggregate Quantities -----------------------------------------------
------------------------------------------------------------------------------------------------
SELECT SalesOrderID, SUM(OrderQty) AS 'Total Quantity Ordered'
	FROM Sales.SalesOrderDetail
	GROUP BY SalesOrderID
	ORDER BY SalesOrderID;
GO

------------------------------------------------------------------------------------------------
--Use Grouping Functions to Aggregate Expressions ----------------------------------------------
------------------------------------------------------------------------------------------------
SELECT MIN(Name) AS 'First Department Name', MAX(Name) AS 'Last Department Name' 
	FROM HumanResources.Department;
GO

/*
------------------------------------------------------------------------------------------------
--Using Multiple Grouping Sets -----------------------------------------------------------------
*/
------------------------------------------------------------------------------------------------
-------------Using GROUP BY with GROUPING SETS Clause-------------------------------------------
USE AdventureWorks2012;
GO
SELECT ST.[Group] AS N'Sales Territory', ST.CountryRegionCode AS N'Country'
    ,S.Name AS N'Store Name', SOH.SalesPersonID AS N'Sales Person ID'
    ,SUM(TotalDue) AS N'Total Sales'
FROM Sales.Customer C
    INNER JOIN Sales.Store AS S
        ON C.StoreID  = S.BusinessEntityId 
    INNER JOIN Sales.SalesTerritory AS ST
        ON C.TerritoryID  = ST.TerritoryID 
    INNER JOIN Sales.SalesOrderHeader AS SOH
        ON C.CustomerID = SOH.CustomerID
WHERE ST.[Group] = N'North America'
    AND ST.CountryRegionCode IN(N'US', N'CA')
    AND SOH.SalesPersonID IN(275, 277, 281, 289)
    AND SUBSTRING(S.Name,1,2)IN(N'Ac', N'Bi ')
GROUP BY GROUPING SETS
    (
	  ST.[Group]
	, ST.CountryRegionCode
	, S.Name
	, SOH.SalesPersonID
	)
ORDER BY ST.[Group], ST.CountryRegionCode, S.Name, SOH.SalesPersonID;
GO

------------------------------------------------------------------------------------------------
-------------Using GROUP BY with Multiple GROUPING SETS ----------------------------------------
USE AdventureWorks2012;
GO
SELECT ST.[Group] AS N'Sales Territory', ST.CountryRegionCode AS N'Country'
    ,S.Name AS N'Store Name', SOH.SalesPersonID AS N'Sales Person ID'
    ,SUM(TotalDue) AS N'Total Sales'
FROM Sales.Customer C
    INNER JOIN Sales.Store AS S
        ON C.StoreID  = S.BusinessEntityId 
    INNER JOIN Sales.SalesTerritory AS ST
        ON C.TerritoryID  = ST.TerritoryID 
    INNER JOIN Sales.SalesOrderHeader AS SOH
        ON C.CustomerID = SOH.CustomerID
WHERE ST.[Group] = N'North America'
    AND ST.CountryRegionCode IN(N'US', N'CA')
    AND SOH.SalesPersonID IN(275, 277, 281, 289)
    AND SUBSTRING(S.Name,1,2)IN(N'Ac', N'Bi ')
GROUP BY GROUPING SETS
(
	(ST.[Group], ST.CountryRegionCode)
	, (ST.[Group], SOH.SalesPersonID )
	, (S.Name                        )
	, (SOH.SalesPersonID             )
	,(                               )
	)
ORDER BY ST.[Group], ST.CountryRegionCode, S.Name, SOH.SalesPersonID;
GO

------------------------------------------------------------------------------------------------
-------------Using ROLLUP Clause with GROUPING SETS ---------------------------------------------

USE AdventureWorks2012;
GO
SELECT ST.[Group] AS N'Sales Territory', ST.CountryRegionCode AS N'Country'
    ,S.Name AS N'Store Name', SOH.SalesPersonID AS N'Sales Person ID'
    ,SUM(TotalDue) AS N'Total Sales'
FROM Sales.Customer C
    INNER JOIN Sales.Store AS S
        ON C.StoreID  = S.BusinessEntityId 
    INNER JOIN Sales.SalesTerritory AS ST
        ON C.TerritoryID  = ST.TerritoryID 
    INNER JOIN Sales.SalesOrderHeader AS SOH
        ON C.CustomerID = SOH.CustomerID
WHERE ST.[Group] = N'North America'
    AND ST.CountryRegionCode IN(N'US', N'CA')
    AND SOH.SalesPersonID IN(275, 277, 281, 289)
    AND SUBSTRING(S.Name,1,2)IN(N'Ac', N'Bi ')
GROUP BY GROUPING SETS
(
	ST.[Group]
	, ST.CountryRegionCode 
	, ROLLUP(S.Name, SOH.SalesPersonID)
	)
ORDER BY ST.[Group], ST.CountryRegionCode, S.Name, SOH.SalesPersonID;
GO

------------------------------------------------------------------------------------------------
-------------Using CUBE Clause with GROUPING SETS ----------------------------------------------
USE AdventureWorks2012;
GO
SELECT ST.[Group] AS N'Sales Territory', ST.CountryRegionCode AS N'Country'
    ,S.Name AS N'Store Name', SOH.SalesPersonID AS N'Sales Person ID'
    ,SUM(TotalDue) AS N'Total Sales'
FROM Sales.Customer C
    INNER JOIN Sales.Store AS S
        ON C.StoreID  = S.BusinessEntityId 
    INNER JOIN Sales.SalesTerritory AS ST
        ON C.TerritoryID  = ST.TerritoryID 
    INNER JOIN Sales.SalesOrderHeader AS SOH
        ON C.CustomerID = SOH.CustomerID
WHERE ST.[Group] = N'North America'
    AND ST.CountryRegionCode IN(N'US', N'CA')
    AND SOH.SalesPersonID IN(275, 277, 281, 289)
    AND SUBSTRING(S.Name,1,2)IN(N'Ac', N'Bi ')
GROUP BY GROUPING SETS
(
	ST.[Group]
	, ST.CountryRegionCode 
	, CUBE(S.Name, SOH.SalesPersonID)
	)
ORDER BY ST.[Group], ST.CountryRegionCode, S.Name, SOH.SalesPersonID;

------------------------------------------------------------------------------------------------
-------------Using ROLLUP Clause AND GROUPING Function ---------------------------------------------------------------
USE AdventureWorks2012;
GO
SELECT ST.[Group] AS N'Sales Territory', ST.CountryRegionCode AS N'Country'
    ,S.Name AS N'Store Name', SOH.SalesPersonID AS N'Sales Person ID'
    ,SUM(TotalDue) AS N'Total Sales', 
	GROUPING(ST.[Group]) AS N'Grouping Sales Territory',
	GROUPING(ST.CountryRegionCode) AS N'Grouping Country'
FROM Sales.Customer C
    INNER JOIN Sales.Store AS S
        ON C.StoreID  = S.BusinessEntityId 
    INNER JOIN Sales.SalesTerritory AS ST
        ON C.TerritoryID  = ST.TerritoryID 
    INNER JOIN Sales.SalesOrderHeader AS SOH
        ON C.CustomerID = SOH.CustomerID
WHERE ST.[Group] = N'North America'
    AND ST.CountryRegionCode IN(N'US', N'CA')
    AND SOH.SalesPersonID IN(275, 277, 281, 289)
    AND SUBSTRING(S.Name,1,2)IN(N'Ac', N'Bi ')
GROUP BY GROUPING SETS
(
	ST.[Group]
	, ST.CountryRegionCode 
	, ROLLUP(S.Name, SOH.SalesPersonID)
	)
ORDER BY ST.[Group], ST.CountryRegionCode, S.Name, SOH.SalesPersonID;
GO