/* Lesson 8.7 Demo 2 -  Use Window Functions in Queries */
---------------------------------------------------------

-----------------------------------------------------------------------------------
--Using Window Aggregate Functions with OVER Clause--------------------------------
-----------------------------------------------------------------------------------
USE AdventureWorks2012;
GO

SELECT ShipMethodID, YEAR(OrderDate) AS N'Year Ordered', COUNT(*) AS 'Number of Orders'
	,SUM(COUNT(*)) OVER(PARTITION BY ShipMethodID) AS 'Total Orders per ShipMethod'
	,AVG(COUNT(*)) OVER(PARTITION BY ShipMethodID) AS 'Average Orders per ShipMethod'
	,MIN(COUNT(*)) OVER(PARTITION BY ShipMethodID) AS 'Minimum Orders per ShipMethod'
	,MAX(COUNT(*)) OVER(PARTITION BY ShipMethodID) AS 'Maximum Orders per ShipMethod'
	,SUM(COUNT(*)) OVER() AS 'Grand Total of Orders'
	FROM  Sales.SalesOrderHeader
	GROUP BY ShipMethodID, YEAR(OrderDate)
	ORDER BY ShipMethodID, YEAR(OrderDate);
GO


-----------------------------------------------------------------------------------
--Using Window Framing Clause with OVER and Frame Unit ROWS------------------------
-- and Delimiters UNBOUNDED PRECEDING----------------------------------------------
-----------------------------------------------------------------------------------
USE AdventureWorks2012;
GO

SELECT CustomerID, SalesOrderID
	,Convert(Date,OrderDate) AS 'Order Date'
	, SubTotal
	,SUM(SubTotal) OVER(PARTITION BY CustomerID
		ORDER BY OrderDate, SalesOrderID
			ROWS BETWEEN UNBOUNDED PRECEDING
				AND CURRENT ROW) AS 'Running Total'
	FROM  Sales.SalesOrderHeader
	Where CustomerID IN ( 11000, 11001, 11002, 11003);
GO

-----------------------------------------------------------------------------------
--Using Window Framing Clause with OVER and Frame Unit ROWS------------------------
-- and Delimiters UNBOUNDED PRECEDING----------------------------------------------
---Filtering AccumulatingTotals to show results of those that are over 5000.00--------
USE AdventureWorks2012;
GO

WITH LargeRunningTotals AS
(
SELECT CustomerID, SalesOrderID
	,Convert(Date,OrderDate) AS 'Order Date'
	, SubTotal
	,SUM(SubTotal) OVER(PARTITION BY CustomerID
		ORDER BY OrderDate, SalesOrderID
			ROWS BETWEEN UNBOUNDED PRECEDING
				AND CURRENT ROW) AS AccumulatingTotal
	FROM  Sales.SalesOrderHeader
	WHERE CustomerID IN ( 11000, 11001, 11002, 11003)
)
SELECT *
	FROM LargeRunningTotals
	WHERE AccumulatingTotal > 5000.00;
GO

-----------------------------------------------------------------------------------
--Using Window Ranking Functions---------------------------------------------------
-----------------------------------------------------------------------------------
USE AdventureWorks2012;
GO

SELECT CustomerID, SalesOrderID, SubTotal
  ,RANK()       OVER(ORDER BY SubTotal) AS Rnk
  ,DENSE_RANK() OVER(ORDER BY SubTotal) AS DenseRnk
  ,NTILE(5)   OVER(ORDER BY SubTotal) AS Ntile5
  ,ROW_NUMBER() OVER(ORDER BY SubTotal) AS RowNum
FROM Sales.SalesOrderHeader
	WHERE CustomerID IN ( 11000, 11001, 11002, 11003, 11004)
	ORDER BY SubTotal, CustomerID, SalesOrderID;
GO

-----------------------------------------------------------------------------------
--Using Window Offset Functions---------------------------------------------------
-----------------------------------------------------------------------------------
--Using LAG to retrieve value from previous row------------------------------------
--Using LEAD to retrieve value from next row---------------------------------------
USE AdventureWorks2012;
GO

SELECT CustomerID, SalesOrderID, SubTotal,
	LAG(SubTotal) OVER(PARTITION BY CustomerID
		ORDER BY OrderDate, SalesOrderID) AS previous_value,
	LEAD(SubTotal) OVER(PARTITION BY CustomerID
		ORDER BY OrderDate, SalesOrderID) AS next_value
FROM Sales.SalesOrderHeader
	WHERE CustomerID IN ( 11000, 11001, 11002, 11003, 11004)
	ORDER BY CustomerID, SalesOrderID, SubTotal;
GO

--Using FIRST_VALUE to retrieve value from first row in window frame 
--Using LAST_VALUE to retrieve value from last row in window frame
USE AdventureWorks2012;
GO
SELECT CustomerID, SalesOrderID, SubTotal,
  FIRST_VALUE(SubTotal)  OVER(PARTITION BY CustomerID
                         ORDER BY OrderDate, SalesOrderID
                         ROWS BETWEEN UNBOUNDED PRECEDING
                                  AND CURRENT ROW) AS first_value,
  LAST_VALUE(SubTotal) OVER(PARTITION BY CustomerID
                       ORDER BY OrderDate, SalesOrderID
                       ROWS BETWEEN CURRENT ROW
                                AND UNBOUNDED FOLLOWING) AS last_value
FROM Sales.SalesOrderHeader
	WHERE CustomerID IN ( 11000, 11001, 11002, 11003, 11004)
	ORDER BY CustomerID, SalesOrderID, SubTotal;
GO

------------------------------------------------------------------------------------
--Using Window Offset Functions - A Practical Application---------------------------
------------------------------------------------------------------------------------
--Using LEAD------------------------------------------------------------------------
--find the difference between a customer's current order and their previous order---
USE AdventureWorks2012;
GO

SELECT CustomerID, SalesOrderID, SubTotal AS 'Current Order',
	SubTotal - LAG(SubTotal) OVER(PARTITION BY CustomerID
		ORDER BY OrderDate, SalesOrderID) AS 'Difference from Previous Order'
FROM Sales.SalesOrderHeader
	WHERE CustomerID IN ( 11000, 11001, 11002, 11003, 11004)
	ORDER BY CustomerID, SalesOrderID, SubTotal;
GO